use axum::{response::{IntoResponse, Response}, http::StatusCode, Json};
use serde::Serialize;

pub type AppResult<T, E = AppError> = std::result::Result<T, E>;

// Make our own error that wraps `anyhow::Error`.
pub struct AppError(pub anyhow::Error);

#[derive(Serialize)]
struct ErrorResponse {
    error: String,
}

// Tell axum how to convert `AppError` into a response.
impl IntoResponse for AppError {
    fn into_response(self) -> Response {
        (
            StatusCode::BAD_REQUEST,
            Json(ErrorResponse { error: format!("{:#}", self.0) }),
        )
            .into_response()
    }
}

// This enables using `?` on functions that return `Result<_, anyhow::Error>` to turn them into
// `Result<_, AppError>`. That way you don't need to do that manually.
impl<E> From<E> for AppError
    where
        E: Into<anyhow::Error>,
{
    fn from(err: E) -> Self {
        Self(err.into())
    }
}
